local REQ_VERSION = 10103

if v and type(v) == "table" then
    v.vfail = false
else
    v = { vfail = true }
end

v.callf = false
v.callT = -1
v.bad = false
v.out = false

local function schedule(f, t)
    assert(not v.callf)
    v.callf = f
    v.callT = t or 0
end

local function failure(err)
    debugLog("VERSION CHECK FAILED: " .. tostring(err))
    centerText(getStringBank(3104))
    v.bad = true
    playSfx("death")
    schedule(goToTitle, 7)
    overrideZoom(1)
    local n = getNaija()
    entity_setState(n, 1234)
    entity_offset(n, 0, 70)
    entity_animate(n, "deadfloat", -1, 0, -1)
    setNaijaHeadTexture("pain", 99)
    bone_alpha(entity_getBoneByName(n, "Cape"), 0)
    cam_snap()
end

local function success(f)
    local s = "Loading and decrunching..."
    local t = createDebugText(s, 12, 400-120, 300-50)
    
    local function line(x)
        s = s .. "\n" .. x
        text_setText(t, s)
    end
    
    local function proceed()
        if isNested() then -- keep trying until Game::applyState() was exited
            schedule(proceed, FRAME_TIME)
        else
            dofile("src/modinit.lua")
            line("Loading map...")
        end
    end
    
    local function boot()
        f()
        line("Success!")
        schedule(proceed)
    end
    
    schedule(boot, FRAME_TIME)
end

local function check(me)
    if not AQUARIA_VERSION then
        return false, "Unknown version (pre-opensource)"
    elseif AQUARIA_VERSION < REQ_VERSION then
        return false, "Outdated version: " .. AQUARIA_VERSION
    end
    
    if v.vfail then
        return false, "Old script interface present"
    end
    
    if not rawget(_G, "text_getStringWidth")
    or not rawget(_G, "node_setActivationRange")
    or not rawget(_G, "isSkippingCutscene")
    or not rawget(_G, "entity_swimToNodeSpeed")
    or not rawget(_G, "createArialTextSmall")
    then -- check for newest function required
        return false, "Too old version: required functions missing"
    end

    local f, err = loadfile("stage0.lua")
    if not f then
        return false, "Loading stage0 failed: " .. tostring(err)
    end
    
    return true, f
end

function init(me)
    debugLog("Version check start...")
    local ok, what = check(me);
    ((ok and success) or failure)(what)
end

function update(me, dt)
    if v.bad then
        disableInput()
        if not v.out and not isNested() and (isEscapeKey() or isLeftMouse() or isRightMouse()) then
            v.out = true
            goToTitle()
        end
    else
        fade(1,0,0,0,0)
    end
    if v.callT >= 0 then
        v.callT = v.callT - dt
        if v.callT <= 0 then
            local f = v.callf
            v.callf = false
            f()
        end
    end
end
