
local T = {}

-- item lookup tables should be all lowercase!

local ITEMNAMES =
{
    -- TODO: item/ingredient names
    naijacorruptedcostume = "Corrupted costume",
    lisuit = "Diving Suit",
    throwbiolight = "Glowfruit",
    naijasuitpartcape = "Naija's cape",
    naijasuitpartleg = "Part of Naija's suit",
    naijasuitpartarm = "Part of Naija's suit",
    naijasuitpartfoot = "Part of Naija's suit",
    throwjelly1 = "Red jelly",
    throwjelly2 = "Yellow jelly",
    throwjelly3 = "Green jelly",
    throwjelly4 = "Orange jelly",
    throwjelly5 = "Blue jelly",
    throwjelly6 = "Purple jelly",
    glowingsoup = "Glowing Soup",

    -- TODO: group names
    --[[perogi = "Perogi",
    meat = "Meat",
    mushroom = "Mushroom",
    oil = "Oil",
    loaf = "Loaf",
    soup = "Soup",]]
}

local ITEMDESC =
{
    weakrainbowsoup = "Makes the world feel slower...",
    strongbooze = "Inebriated",
    testobject = "Slightly inebriated",
    
    naijacorruptedcostume = "Pain becomes rage,\nbut at a price...",
    lisuit = "Allows Li to swim reasonably fast",
    throwbiolight = "Glows in the dark when carried",
    naijasuitpartcape = "She would never have left it behind...\n \n(Use to equip)",
    naijasuitpartleg = "",
    naijasuitpartarm = "",
    naijasuitpartfoot = "",
    
    throwjelly1 = "Anemone bait",
    throwjelly2 = "Anemone bait",
    throwjelly3 = "Anemone bait",
    throwjelly4 = "Anemone bait",
    throwjelly5 = "Anemone bait",
    throwjelly6 = "Anemone bait",

}

local ITEMFAIL =
{
    max = "Have enough of these!",
    error = "Programming error -- missing data?",
    weight = "Weight exceeded!",
    fullhealth = "Already at full health!",
    dropimportant = "Too important to drop!",
    cookimportant = "Too important to waste!",
    cantcook = "Can't cook this!",
    maxbox = "Box would burst!", -- the first person complaining about getting this one gets shot
}



local function lookupNoCase(t, k)
    return rawget(t, k:lower())
end

setmetatable(ITEMNAMES, { __index = lookupNoCase })
setmetatable(ITEMDESC,  { __index = lookupNoCase })

local function multAsPercInv(x)
    local p = 100*(1-x)
    if p < 0 then
        return p .. "%"
    end
    return "+" .. p .. "%"
end

local function multAsPerc(x)
    local p = 100*(x-1)
    if p < 0 then
        return p .. "%"
    end
    return "+" .. p .. "%"
end

local POISON_LEVEL_STR =
{
    [0] = "weak",
    [1] = "medium",
    [2] = "strong",
    [3] = "very strong",
}

local AURACLEAR =
{
    blind = "Cures blindness",
}


--[[local STATS_NAMES =
{
    kills = "Total kills",
}]]

local STRINGBUILDER =
{
    -- auras
    speed         = function(time, val) return "Speed " .. multAsPerc(val) .. " for " .. time .. " seconds" end,
    attackpower   = function(time, val) return "Attack power +" .. val .. " for " .. time .. " seconds" end,
    defense       = function(time, val) return "Defense " .. multAsPercInv(val) .. " for " .. time .. " seconds" end,
    regen         = function(time, val) return "Regeneration +" .. val .. " for " .. time .. " seconds" end,
    trip          = function()          return "Trippy!" end,
    antipoison    = function(time, val)
        local str = "<unk>"
        if type(val) == "number" then
            str = POISON_LEVEL_STR[val] or str
        end
        return "Immunity against " .. str .. " poison for ".. time .. " seconds"
    end,
    
    -- other
    heal          = function(val)
        if val < 99 then
            return "Heal +" .. val
        else
            return "Full health"
        end
    end,
    
    addpoison = function() return "Poisoned" end,
    clearaura = function(val)
        return AURACLEAR[val]
    end,
    throwable = function() return "Can be thrown" end,
    clearpoison = function(val) return "Cures " .. tostring(POISON_LEVEL_STR[val]) .. "poison" end,
}

local FLAT_VALUES = { "heal", }

local function buildAuraString(auradata)
    local a, tm, val = unpack(auradata)
    local f = STRINGBUILDER[a]
    if f then
        return f(tm, val)
    else
        return "[aura:" .. a .. "]: " .. tostring(tm) .. ", " .. tostring(val)
    end
end

local function buildMiscString(data, category)
    local f = STRINGBUILDER[category]
    if f then
        return f(data)
    else
        return "[misc:" .. tostring(data) .. "]"
    end
end

local function buildSingle(f, to, from, ...)
    if from then
        table.insert(to, f(from, ...))
    end
end

local function buildMultiple(f, to, from, ...)
    if from then
        for _, x in ipairs(from) do
            table.insert(to, f(x, ...))
        end
    end
end

local function buildItemEffectString(itemname)
    local itm = DATA_ITEMS[itemname]
    if not itm then return end
    
    local tab = {}
    for _, valname in ipairs(FLAT_VALUES) do
        local val = itm[valname]
        if val then
            local f = STRINGBUILDER[valname]
            if f then
                table.insert(tab, f(val))
            else
                table.insert(tab, "[" .. valname .. "]: " .. val)
            end
        end
    end
    buildSingle(buildAuraString, tab, itm.aura)
    buildMultiple(buildAuraString, tab, itm.auras)
    buildSingle(buildMiscString, tab, itm.clearaura, "clearaura")
    buildMultiple(buildMiscString, tab, itm.clearauras, "clearaura")
    buildSingle(buildMiscString, tab, itm.poison, "poison")
    buildSingle(buildMiscString, tab, itm.throwable, "throwable")
    
    return table.concat(tab, "\n")
end

local function _splitCamelCase(c)
    return " " .. c
end

function T.getItemName(s)
    local x = ITEMNAMES[s] or s:gsub("[A-Z]", _splitCamelCase)
    if x:startsWith(" ") then
        x = x:sub(2)
    end
    return x
end

local function getDefaultDesc(s)
    local itm = DATA_ITEMS[s]
    if not itm then return end
    
    if itm.type == "food" then
        return "" --"Cooking ingredient"
    end
end

function T.getItemDesc(s)
    local desc = ITEMDESC[s]
    local eff = buildItemEffectString(s)
    if not desc and (not eff or eff == "") then
        desc = getDefaultDesc(s)
    end
    if desc then
        desc =  desc .. "\n" .. eff
    else
        desc = eff
    end
    if desc == "" then
        desc = nil
    end
    return desc
end

function T.getItemFailureMessage(fail, name, rawname)
    local fmt = ITEMFAIL[fail]
    if fmt then
        return fmt:format(name)
    end
    return "Unknown failure [" .. tostring(fail) .. "] for item " .. name
end

return T

